@@ -56,8 +56,8 @@ RcppExport SEXP RtoSmryCpp(SEXP params, SEXP Ysexp, SEXP Xsexp,
     }
        
     // do stuff	
-  clock_t clk_start, clk_end;
-  clk_start = clock();
+//  clock_t clk_start, clk_end;
+//  clk_start = clock();
 
     // initialize summary class
     Summary smry(&mm, Y, X);
