@@ -6,193 +6,178 @@
 
 int testStatCalc(mv_mat *H0, mv_mat *H1, mv_Method *mmRef, const unsigned int ifcalcH1det, double *stat, gsl_vector *statj)
 {
-	unsigned int i, j;
-        int s;
+	unsigned int j;
         unsigned int nVars=H0->SS->size1;
         unsigned int nRows=H0->mat->size1;
+	unsigned int nP0=H0->X->size2;
+	unsigned int nP1=H1->X->size2;
+	double scale = (double)(nRows-nP1)/(double)(nP1-nP0);
 	double ss0j, ss1j, sum;
         double logDetSS0, logDetSS1;
+	gsl_matrix *SS0 = gsl_matrix_alloc(nVars, nVars);
+	gsl_matrix *SS1 = gsl_matrix_alloc(nVars, nVars);
+	gsl_matrix *Sd = gsl_matrix_alloc(nVars, nVars);
 
-	// calc test stats
-	if ( mmRef->test == LOGWILK ) {
-	   // statj = nRows*(log(diag(ss0)) - log(diag(ss1)));
-	    for ( j=0; j<nVars; j++ ) {
-                 ss0j = gsl_matrix_get(H0->SS, j, j);
-                 ss1j = gsl_matrix_get(H1->SS, j, j);
-	         gsl_vector_set(statj, j, nRows*(log(ss0j)-log(ss1j)));
-	    } 
-            if ( mmRef->corr == IDENTITY ){ 
-            	 sum = 0.0;
-                 for ( j=0; j<nVars; j++ )
-	             sum += gsl_vector_get(statj, j);
+        // univariate test
+	for ( j=0; j<nVars; j++ ) {
+            ss0j = gsl_matrix_get(H0->SS, j, j);
+            ss1j = gsl_matrix_get(H1->SS, j, j);
+	    if ( mmRef->test == LOGWILK ) 
+	        gsl_vector_set(statj, j, nRows*(log(ss0j)-log(ss1j)));
+            else // HOTELLING 
+                gsl_vector_set(statj, j, (double)scale*(ss0j-ss1j)/ss1j);
+	} 
+
+	// multivariate test
+        if ( mmRef->corr == IDENTITY ){ 
+            sum = 0.0;
+	    for (j=0; j<nVars;j++) sum += gsl_vector_get(statj, j);
+        }
+        else {
+	    if (mmRef->corr == SHRINK) {
+               gsl_matrix_memcpy(SS0, H0->R);
+	       gsl_matrix_set_zero (Sd);
+	       gsl_blas_dger (1.0, H0->sd, H0->sd, Sd);
+	       gsl_matrix_mul_elements(SS0, Sd);
+	       gsl_matrix_memcpy(SS1, H1->R);
+	       gsl_matrix_set_zero (Sd);
+	       gsl_blas_dger (1.0, H1->sd, H1->sd, Sd);
+	       gsl_matrix_mul_elements(SS1, Sd);
             }
 	    else {
-	      	 logDetSS0 = log( calcDet(H0->SS) );		 
-                 //printf("%4.2f ", *logDetSS0);
+	       gsl_matrix_memcpy(SS0, H0->SS);
+	       gsl_matrix_memcpy(SS1, H1->SS);
+	    }
+
+            if (mmRef->test == LOGWILK) {
+	      	logDetSS0 = logDet(SS0) ;		 
 		 // used in summary to speed up 
 	         if (ifcalcH1det == TRUE){
-	            logDetSS1 = log( calcDet(H1->SS) );
+	            logDetSS1 = logDet(SS1);
 		    H1->teststat = logDetSS1;
 		 }
-		 else
-		    logDetSS1 = H1->teststat;
+		 else logDetSS1 = H1->teststat;
                  sum = nRows * (logDetSS0 - logDetSS1);
             }
-	}
-	else if ( mmRef->test == HOTELING ) { // Hotelling 
-           // statj = diag(ss0)./diag(ss1)-1;  
-	   for ( j=0; j<nVars; j++ ) {
-               ss0j = gsl_matrix_get(H0->SS, j, j);
-	       ss1j = gsl_matrix_get(H1->SS, j, j);
-	       gsl_vector_set(statj, j, (double)ss0j/ss1j-1);
-	   }
-	   if ( mmRef->corr == IDENTITY ){ 
+	    else {
+	       // (SS0-SS1)SS1^-1
+	       gsl_matrix_sub(SS0, SS1);
+	       gsl_blas_dtrsm (CblasRight, CblasLower, CblasNoTrans, CblasNonUnit, 1.0, SS1, SS0);
 	       sum = 0.0;
-               for ( j=0; j<nVars; j++ )
-	           sum += gsl_vector_get(statj, j);
-           }
-	   else {
- 	       gsl_matrix *Sa = gsl_matrix_alloc(nVars, nVars);
-               gsl_matrix *LU=gsl_matrix_alloc(nVars, nVars);
-               gsl_matrix_memcpy(LU, H1->SS); 
-               gsl_matrix_memcpy(Sa, H0->SS);
-               for (i=0;i<nVars; i++)
-               for (j=i+1; j<nVars; j++){ 
-                   gsl_matrix_set(LU, i, j, gsl_matrix_get(H1->SS, j, i));
-                   gsl_matrix_set(Sa, i, j, gsl_matrix_get(H0->SS, j, i)); 
-               }
-               gsl_permutation *p = gsl_permutation_alloc(nVars);
-               gsl_linalg_LU_decomp (LU, p, &s);
-               for (j=0; j<nVars; j++) { 
-                  gsl_vector_view xj=gsl_matrix_column (Sa, j);
-                  gsl_linalg_LU_svx (LU, p, &xj.vector);
-               } 
-	       sum = 0.0;
-	       for ( j=0; j<nVars; j++ ) 
-	           sum += gsl_matrix_get(Sa, j, j)-1;
-               //printf("multstat=%.4f\n", sum);
-
-	       gsl_matrix_free(Sa);
-               gsl_permutation_free(p);
-               gsl_matrix_free(LU);
+	       for (j=0; j<nVars;j++) sum += gsl_matrix_get(SS0, j, j); 
+	       sum = sum*scale;
            }
 	}
-	else {
-	   GSL_ERROR("Invalid test type", GSL_EINVAL);
-	   return GSL_EINVAL;
-	}
-
         // copy results
 	*stat = sum;
 
+	gsl_matrix_free(Sd);
+	gsl_matrix_free(SS0);
+	gsl_matrix_free(SS1);
+
 	return 0;
 }
 
-int calcSS(gsl_matrix *Y, mv_mat *Hat, mv_Method *mmRef, const unsigned int ifcalcHat, const unsigned int ifcalcCoef, const unsigned int ifcalcSS)
+int calcSS(gsl_matrix *Y, mv_mat *Hat, mv_Method *mmRef)
 {
     gsl_set_error_handler_off();
 
-    double mean, dd;	
-    unsigned int j, status; 
+    double dd;	
+    unsigned int j;
+    int status; 
     unsigned int nP=Hat->X->size2;
     unsigned int nRows=Hat->mat->size1;
     unsigned int nVars=Hat->SS->size1;
     gsl_matrix *tXX = gsl_matrix_alloc(nP, nP);
     gsl_matrix *tXXtX = gsl_matrix_alloc(nP, nRows);
-    gsl_matrix *I_Hat = gsl_matrix_alloc(nRows, nRows);
-    gsl_vector *e = gsl_vector_alloc(nRows);
-    gsl_vector_set_all (e, 1.0); 
-
+    gsl_matrix *Sd = gsl_matrix_alloc(nVars, nVars);
      // It is possible later to feed data with more varialbes (columns) 
      // than observations (rows). So better use SVD than QR
      // As in manyglm use (X'X)^-1 X' and cholesky on (X'X) instead 
+    
      gsl_matrix_set_identity(tXX);
      gsl_blas_dsyrk (CblasLower, CblasTrans, 1.0, Hat->X, 0.0, tXX);
+
      status=gsl_linalg_cholesky_decomp(tXX);
      if (status==GSL_EDOM) {
-       // if (mmRef->warning==TRUE) { 
-       //    printf("Warning: Model may be too complexed. Fitting is not reliable\n");
-       // }
         gsl_matrix_set_identity(tXX);
         // add an eps*I to the diagonal matrix
         gsl_blas_dsyrk (CblasLower, CblasTrans, 1.0, Hat->X, TOL, tXX);
         status=gsl_linalg_cholesky_decomp(tXX);
      }
      gsl_linalg_cholesky_invert (tXX);
-     // Hat = X (XX)^-1 t(X) 
+//     gsl_blas_dgemm (CblasTrans, CblasNoTrans, 1.0, Hat->X, Hat->X, 0.0, tXX); 
+//     for (i=0; i<nRows; i++) {
+//         gsl_vector_view hi=gsl_matrix_column(tXXtX, i);
+//         gsl_vector_view xi=gsl_matrix_row(Hat->X, i);
+//         GetCholstat(tXX, &xi.vector, &hi.vector);    
+//     }
+
      gsl_blas_dgemm (CblasNoTrans, CblasTrans, 1.0, tXX, Hat->X, 0.0, tXXtX); 
-     if (ifcalcCoef==TRUE) {
-        gsl_blas_dgemm (CblasNoTrans, CblasTrans, 1.0, tXXtX, Y, 0.0, Hat->Coef); 
-     }   
-     if ( (ifcalcHat==TRUE)|(ifcalcSS==TRUE) ) {
-        gsl_blas_dgemm (CblasNoTrans, CblasNoTrans, 1.0, Hat->X, tXXtX, 0.0, Hat->mat); 
+     gsl_blas_dgemm (CblasNoTrans, CblasTrans, 1.0, tXXtX, Y, 0.0, Hat->Coef); 
+     // Hat = X (XX)^-1 t(X) 
+     gsl_blas_dgemm (CblasNoTrans, CblasNoTrans, 1.0, Hat->X, tXXtX, 0.0, Hat->mat); 
+     // Res = Y - Hat*Y 
+     gsl_blas_dgemm (CblasNoTrans, CblasNoTrans, -1.0, Hat->mat, Y, 0.0, Hat->Res);  
+     gsl_matrix_add (Hat->Res, Y);
+     // subtractMean(Hat->Res);
+     gsl_blas_dgemm(CblasTrans,CblasNoTrans,1.0,Hat->Res,Hat->Res,0.0,Hat->SS);  
+     for (j=0; j<nVars; j++){
+         dd = gsl_matrix_get(Hat->SS, j, j);
+	 if (dd<TOL) dd = 0.5*M_1_PI;
+	 gsl_matrix_set(Hat->SS, j, j, dd);
+	 gsl_vector_set(Hat->sd, j, sqrt(dd));
      }
-     if (ifcalcSS==TRUE) {
-        gsl_matrix_set_identity(I_Hat);
-        gsl_matrix_sub(I_Hat, Hat->mat);   
-        gsl_blas_dgemm (CblasNoTrans, CblasNoTrans, 1.0, I_Hat, Y, 0.0, Hat->Res); 
-        // subtract mean
-        // rcalc:  rcalc(Hat->Res, shrink_param, corr, Hat->SS);
-        gsl_matrix_set_zero(Hat->SS);
-        // subtract mean = Rs^T e(12) from res
-        for ( j=0; j<nVars; j++ ) {
-            gsl_vector_view rj=gsl_matrix_column(Hat->Res, j);
-            gsl_blas_ddot (&rj.vector, e, &mean); 
-            gsl_vector_add_constant (&rj.vector, -mean/nRows); 
-            if ( mmRef->corr == IDENTITY ) {
-               gsl_blas_ddot (&rj.vector, &rj.vector, &dd); 
-               if ( dd < TOL ) dd = 0.5*M_1_PI;
-               gsl_matrix_set(Hat->SS, j, j, dd);
-            }
-         } 
-        // compute the covariance matrix SS= (res'*res)/Rows-1
-         if ( mmRef->corr != IDENTITY ){
-            gsl_blas_dsyrk(CblasLower, CblasTrans,1.0, Hat->Res, 0.0, Hat->SS);
-            gsl_matrix_scale ( Hat->SS, 1.0/(double)(nRows-1) );
-            if ( mmRef->corr == SHRINK ) {
-               gsl_vector_view dj=gsl_matrix_diagonal (Hat->SS);
-               gsl_vector_scale (&dj.vector, 1.0/(mmRef->shrink_param));
-            }
-         } 
+     if (mmRef->corr==IDENTITY) gsl_matrix_set_identity(Hat->R);
+     else {
+         gsl_matrix_memcpy(Hat->R, Hat->SS);
+	 gsl_matrix_set_zero(Sd);
+	 gsl_blas_dger (1.0, Hat->sd, Hat->sd, Sd);
+	 gsl_matrix_div_elements(Hat->R, Sd);
+	 if (mmRef->corr==SHRINK) {
+	    // lambda * R + (1-lambda)*I
+	    gsl_matrix_scale(Hat->R, (double)mmRef->shrink_param);
+	    gsl_vector_view dj=gsl_matrix_diagonal(Hat->R);
+	    gsl_vector_add_constant(&dj.vector, (double)(1.0-mmRef->shrink_param));
+	  }
      }
-     gsl_vector_free(e);
-     gsl_matrix_free(I_Hat);
+ 
+     gsl_matrix_free(Sd);
      gsl_matrix_free(tXX);
      gsl_matrix_free(tXXtX);
 
      return 0;
 }
 
-double calcDet(gsl_matrix *SS)
+double logDet(gsl_matrix *SS)
 {
      // SS is a nVars x nVars real symmetric matric
      unsigned int nVars = SS->size1;
-     double result;
-/*
+
      // det(A) = prod(eig(A))
      gsl_eigen_symm_workspace *ws = gsl_eigen_symm_alloc(nVars);
      gsl_vector *eval=gsl_vector_alloc(nVars);    
      unsigned int j;
      double logDetSS=0.0;
      gsl_eigen_symm(SS, eval, ws);
-     displayvector(eval, "SS eigen values");
+ //    displayvector(eval, "SS eigen values");
      for ( j=0; j<nVars; j++ )
          logDetSS += log(gsl_vector_get(eval, j));
 
      gsl_eigen_symm_free(ws);
      gsl_vector_free(eval);
      return logDetSS;
-*/  
-
+  
+/*
    // OR: det(A) = diag(LU)
      // fill SS
      gsl_matrix *LU=gsl_matrix_alloc(nVars, nVars);
      gsl_matrix_memcpy(LU, SS); 
      unsigned int i, j;
      int s;
-     for (i=0;i<nVars; i++)
-     for (j=i+1; j<nVars; j++)
-         gsl_matrix_set(LU, i, j, gsl_matrix_get(LU, j, i));
+//     for (i=0;i<nVars; i++)
+//     for (j=i+1; j<nVars; j++)
+//         gsl_matrix_set(LU, i, j, gsl_matrix_get(LU, j, i));
 
      gsl_permutation *p = gsl_permutation_alloc(nVars);
      gsl_linalg_LU_decomp (LU, p, &s);
@@ -202,7 +187,7 @@ double calcDet(gsl_matrix *SS)
      gsl_matrix_free(LU);
 
      return result;
-
+*/
 }
 
 int is_sym_matrix(const gsl_matrix *mat)
@@ -508,6 +493,57 @@ int invLSQ(gsl_matrix *A, gsl_vector *b, gsl_vector *x)
    return 0;
 }
 
+
+double GetSVDstat(gsl_matrix *A, gsl_vector *b, gsl_vector *work)
+{ 
+   unsigned int dim = A->size2;
+   unsigned int k;
+   double result;
+
+   gsl_matrix *nullspace = gsl_matrix_alloc(dim, dim);
+   gsl_vector *eigval = gsl_vector_alloc(dim);
+
+   gsl_linalg_SV_decomp (A, nullspace, eigval, work);
+   for (k=0; k<dim; k++) {
+        if (gsl_vector_get(eigval, k) < TOL )
+	    gsl_vector_set(eigval, k, 0);
+   }	    
+   gsl_linalg_SV_solve (A, nullspace, eigval, b, work);
+   gsl_blas_ddot(b, work, &result);
+
+   gsl_vector_free(eigval);
+   gsl_matrix_free(nullspace);
+
+   return result;
+}
+
+
+double GetCholstat(gsl_matrix *A, gsl_vector *b, gsl_vector *work)
+{ 
+   gsl_set_error_handler_off();
+
+   unsigned int dim = A->size1;
+   int status;
+   double result;
+   gsl_matrix *V = gsl_matrix_alloc(dim, dim);
+
+   gsl_matrix_memcpy(V, A);
+   status=gsl_linalg_cholesky_decomp(V);
+   if (status==GSL_EDOM) {
+      gsl_matrix_memcpy(V, A);
+      gsl_vector_view diag_V = gsl_matrix_diagonal(V);
+      gsl_vector_add_constant(&diag_V.vector, TOL);
+      gsl_linalg_cholesky_decomp(V);
+   }
+   // result b^T V^-1 b
+   gsl_linalg_cholesky_solve(V, b, work);
+   gsl_blas_ddot(b, work, &result);
+ 
+   gsl_matrix_free(V);
+
+   return result;
+}
+
 // the manyglm version of rcalc
 int GetR(gsl_matrix *Res, unsigned int corr, double lambda, gsl_matrix *R)
 {
@@ -592,3 +628,4 @@ int subtractMean(gsl_matrix *dat)
     return SUCCESS;
 }
 
+
